@@ -2,6 +2,7 @@ from WoeUSB import \
     core, \
     list_devices, \
     utils, \
-    workaround
+    workaround, \
+    i18n
 
-__version__ = "0.1"
+__version__ = "0.2.6"
